@@ -1,6 +1,6 @@
 /*
  *   stunnel       Universal SSL tunnel
- *   Copyright (C) 1998-2011 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *   Copyright (C) 1998-2012 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -49,8 +49,8 @@
 #define DEFAULT_STACK_SIZE 65536
 /* #define DEBUG_STACK_SIZE */
 
-/* I/O buffer size */
-#define BUFFSIZE 8192
+/* I/O buffer size - 18432 is the maximum size of SSL record payload */
+#define BUFFSIZE 18432
 
 /* how many bytes of random input to read from files for PRNG */
 /* OpenSSL likes at least 128 bits, so 64 bytes seems plenty. */
@@ -72,7 +72,6 @@ typedef int socklen_t;
 
 #ifdef USE_WIN32
 #define USE_IPv6
-/* #define USE_FIPS */
 #define _CRT_SECURE_NO_DEPRECATE
 #define _CRT_NONSTDC_NO_DEPRECATE
 #define HAVE_OSSL_ENGINE_H
@@ -85,42 +84,44 @@ typedef int socklen_t;
 #define S_EADDRINUSE  WSAEADDRINUSE
 /* winsock does not define WSAEAGAIN */
 /* in most (but not all!) BSD implementations EAGAIN==EWOULDBLOCK */
-#define S_EAGAIN      WSAEWOULDBLOCK
-#define S_ECONNRESET  WSAECONNRESET
-#define S_EINPROGRESS WSAEINPROGRESS
-#define S_EINTR       WSAEINTR
-#define S_EINVAL      WSAEINVAL
-#define S_EISCONN     WSAEISCONN
-#define S_EMFILE      WSAEMFILE
+#define S_EAGAIN        WSAEWOULDBLOCK
+#define S_ECONNRESET    WSAECONNRESET
+#define S_EINPROGRESS   WSAEINPROGRESS
+#define S_EINTR         WSAEINTR
+#define S_EINVAL        WSAEINVAL
+#define S_EISCONN       WSAEISCONN
+#define S_EMFILE        WSAEMFILE
 /* winsock does not define WSAENFILE */
-#define S_ENOBUFS     WSAENOBUFS
+#define S_ENOBUFS       WSAENOBUFS
 /* winsock does not define WSAENOMEM */
-#define S_ENOPROTOOPT WSAENOPROTOOPT
-#define S_ENOTSOCK    WSAENOTSOCK
-#define S_EOPNOTSUPP  WSAEOPNOTSUPP
-#define S_EWOULDBLOCK WSAEWOULDBLOCK
+#define S_ENOPROTOOPT   WSAENOPROTOOPT
+#define S_ENOTSOCK      WSAENOTSOCK
+#define S_EOPNOTSUPP    WSAEOPNOTSUPP
+#define S_EWOULDBLOCK   WSAEWOULDBLOCK
+#define S_ECONNABORTED  WSAECONNABORTED
 #else /* USE_WIN32 */
-#define S_EADDRINUSE  EADDRINUSE
-#define S_EAGAIN      EAGAIN
-#define S_ECONNRESET  ECONNRESET
-#define S_EINPROGRESS EINPROGRESS
-#define S_EINTR       EINTR
-#define S_EINVAL      EINVAL
-#define S_EISCONN     EISCONN
-#define S_EMFILE      EMFILE
+#define S_EADDRINUSE    EADDRINUSE
+#define S_EAGAIN        EAGAIN
+#define S_ECONNRESET    ECONNRESET
+#define S_EINPROGRESS   EINPROGRESS
+#define S_EINTR         EINTR
+#define S_EINVAL        EINVAL
+#define S_EISCONN       EISCONN
+#define S_EMFILE        EMFILE
 #ifdef ENFILE
-#define S_ENFILE      ENFILE
+#define S_ENFILE        ENFILE
 #endif
 #ifdef ENOBUFS
-#define S_ENOBUFS     ENOBUFS
+#define S_ENOBUFS       ENOBUFS
 #endif
 #ifdef ENOMEM
-#define S_ENOMEM      ENOMEM
+#define S_ENOMEM        ENOMEM
 #endif
-#define S_ENOPROTOOPT ENOPROTOOPT
-#define S_ENOTSOCK    ENOTSOCK
-#define S_EOPNOTSUPP  EOPNOTSUPP
-#define S_EWOULDBLOCK EWOULDBLOCK
+#define S_ENOPROTOOPT   ENOPROTOOPT
+#define S_ENOTSOCK      ENOTSOCK
+#define S_EOPNOTSUPP    EOPNOTSUPP
+#define S_EWOULDBLOCK   EWOULDBLOCK
+#define S_ECONNABORTED  ECONNABORTED
 #endif /* USE_WIN32 */
 
 /**************************************** generic headers */
@@ -174,6 +175,7 @@ typedef int socklen_t;
 #include <time.h>
 #include <sys/stat.h>    /* stat */
 #include <setjmp.h>
+#include <fcntl.h>
 
 /**************************************** WIN32 headers */
 
@@ -183,18 +185,21 @@ typedef unsigned char u8;
 typedef unsigned short u16;
 typedef unsigned long u32;
 
+#define HAVE_STRUCT_ADDRINFO
 #define HAVE_SNPRINTF
-#define snprintf _snprintf
+#define snprintf                    _snprintf
 #define HAVE_VSNPRINTF
-#define vsnprintf _vsnprintf
-#define strcasecmp _stricmp
-#define strncasecmp _strnicmp
-#define sleep(c) Sleep(1000*(c))
+#define vsnprintf                   _vsnprintf
+#define strcasecmp                  _stricmp
+#define strncasecmp                 _strnicmp
+#define sleep(c)                    Sleep(1000*(c))
 
-#define get_last_socket_error() WSAGetLastError()
-#define get_last_error()        GetLastError()
-#define readsocket(s,b,n)       recv((s),(b),(n),0)
-#define writesocket(s,b,n)      send((s),(b),(n),0)
+#define get_last_socket_error()     WSAGetLastError()
+#define set_last_socket_error(e)    WSASetLastError(e)
+#define get_last_error()            GetLastError()
+#define set_last_error(e)           SetLastError(e)
+#define readsocket(s,b,n)           recv((s),(b),(n),0)
+#define writesocket(s,b,n)          send((s),(b),(n),0)
 
 /* #define FD_SETSIZE 4096 */
 /* #define Win32_Winsock */
@@ -203,11 +208,14 @@ typedef unsigned long u32;
 /* Winsock2 header for IPv6 definitions */
 #include <winsock2.h>
 #include <ws2tcpip.h>
+
 #include <windows.h>
 
 #include <process.h>     /* _beginthread */
 #include <tchar.h>
 
+#include "resources.h"
+
 /**************************************** non-WIN32 headers */
 
 #else /* USE_WIN32 */
@@ -229,24 +237,28 @@ typedef unsigned long u32;
 #endif
 
 #ifdef __INNOTEK_LIBC__
-#define socklen_t               __socklen_t
-#define strcasecmp              stricmp
-#define strncasecmp             strnicmp
-#define NI_NUMERICHOST          1
-#define NI_NUMERICSERV          2
-#define get_last_socket_error() sock_errno()
-#define get_last_error()        errno
-#define readsocket(s,b,n)       recv((s),(b),(n),0)
-#define writesocket(s,b,n)      send((s),(b),(n),0)
-#define closesocket(s)          close(s)
-#define ioctlsocket(a,b,c)      so_ioctl((a),(b),(c))
+#define socklen_t                   __socklen_t
+#define strcasecmp                  stricmp
+#define strncasecmp                 strnicmp
+#define NI_NUMERICHOST              1
+#define NI_NUMERICSERV              2
+#define get_last_socket_error()     sock_errno()
+#define set_last_socket_error(e)    ()
+#define get_last_error()            errno
+#define set_last_error(e)           (errno=(e))
+#define readsocket(s,b,n)           recv((s),(b),(n),0)
+#define writesocket(s,b,n)          send((s),(b),(n),0)
+#define closesocket(s)              close(s)
+#define ioctlsocket(a,b,c)          so_ioctl((a),(b),(c))
 #else
-#define get_last_socket_error() errno
-#define get_last_error()        errno
-#define readsocket(s,b,n)       read((s),(b),(n))
-#define writesocket(s,b,n)      write((s),(b),(n))
-#define closesocket(s)          close(s)
-#define ioctlsocket(a,b,c)      ioctl((a),(b),(c))
+#define get_last_socket_error()     errno
+#define set_last_socket_error(e)    (errno=(e))
+#define get_last_error()            errno
+#define set_last_error(e)           (errno=(e))
+#define readsocket(s,b,n)           read((s),(b),(n))
+#define writesocket(s,b,n)          write((s),(b),(n))
+#define closesocket(s)              close(s)
+#define ioctlsocket(a,b,c)          ioctl((a),(b),(c))
 #endif
 
     /* OpenVMS compatibility */
@@ -302,7 +314,6 @@ typedef unsigned long u32;
 #ifdef __BEOS__
 #include <posix/grp.h>
 #endif
-#include <fcntl.h>
 
 #include <netinet/in.h>  /* struct sockaddr_in */
 #include <sys/socket.h>  /* getpeername */
@@ -369,6 +380,10 @@ extern char *sys_errlist[];
 #error OpenSSL library compiled without thread support
 #endif /* !OPENSSL_THREADS && USE_PTHREAD */
 
+#if defined (USE_WIN32) && defined(OPENSSL_FIPS)
+#define USE_FIPS
+#endif
+
 /* OpenSSL 0.9.6 comp.h needs ZLIB macro to declare COMP_zlib() */
 #define ZLIB
 
@@ -407,6 +422,11 @@ extern char *sys_errlist[];
 #define OPENSSL_NO_TLSEXT
 #endif /* OpenSSL version < 1.0.0 */
 
+#ifndef OPENSSL_NO_COMP
+/* not defined in public headers before OpenSSL 0.9.8 */
+STACK_OF(SSL_COMP) *SSL_COMP_get_compression_methods(void);
+#endif /* OPENSSL_NO_COMP */
+
 /**************************************** other defines */
 
 /* change all non-printable characters to '.' */
@@ -437,6 +457,10 @@ extern char *sys_errlist[];
 #define LOG_DEBUG       7
 #endif /* defined (USE_WIN32) || defined (__vms) */
 
+#ifndef offsetof
+#define offsetof(T, F) ((unsigned int)((char *)&((T *)0L)->F - (char *)0L))
+#endif
+
 #endif /* defined COMMON_H */
 
 /* end of common.h */
