@@ -1,6 +1,6 @@
 /*
  *   stunnel       Universal SSL tunnel
- *   Copyright (C) 1998-2011 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *   Copyright (C) 1998-2012 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -50,7 +50,7 @@ DISK_FILE *file_open(char *name, int wr) {
     fh=CreateFile(tstr, wr ? GENERIC_WRITE : GENERIC_READ,
         FILE_SHARE_READ, NULL, wr ? OPEN_ALWAYS : OPEN_EXISTING,
         FILE_ATTRIBUTE_NORMAL, (HANDLE)NULL);
-    free(tstr);
+    str_free(tstr);
     if(fh==INVALID_HANDLE_VALUE) {
         ioerror(name);
         return NULL;
@@ -59,11 +59,7 @@ DISK_FILE *file_open(char *name, int wr) {
         SetFilePointer(fh, 0, NULL, FILE_END);
 
     /* setup df structure */
-    df=calloc(1, sizeof df);
-    if(!df) {
-        CloseHandle(df->fh);
-        return NULL;
-    }
+    df=str_alloc(sizeof df);
     df->fh=fh;
     return df;
 }
@@ -73,9 +69,7 @@ DISK_FILE *file_open(char *name, int wr) {
 DISK_FILE *file_fdopen(int fd) {
     DISK_FILE *df;
 
-    df=calloc(1, sizeof(DISK_FILE));
-    if(!df)
-        return NULL;
+    df=str_alloc(sizeof(DISK_FILE));
     df->fd=fd;
     return df;
 }
@@ -105,11 +99,7 @@ DISK_FILE *file_open(char *name, int wr) {
     }
 
     /* setup df structure */
-    df=calloc(1, sizeof df);
-    if(!df) {
-        close(df->fd);
-        return NULL;
-    }
+    df=str_alloc(sizeof df);
     df->fd=fd;
     return df;
 }
@@ -124,7 +114,7 @@ void file_close(DISK_FILE *df) {
 #else /* USE_WIN32 */
     close(df->fd);
 #endif /* USE_WIN32 */
-    free(df);
+    str_free(df);
 }
 
 int file_getline(DISK_FILE *df, char *line, int len) {
@@ -171,9 +161,7 @@ int file_putline(DISK_FILE *df, char *line) {
 #endif /* USE_WIN32 */
 
     len=strlen(line);
-    buff=calloc(len+2, 1); /* +2 for CR+LF */
-    if(!buff)
-        return 0;
+    buff=str_alloc(len+2); /* +2 for CR+LF */
     strcpy(buff, line);
 #ifdef USE_WIN32
     buff[len++]='\r'; /* CR */
@@ -185,7 +173,7 @@ int file_putline(DISK_FILE *df, char *line) {
     /* no file -> write to stderr */
     num=write(df ? df->fd : 2, buff, len);
 #endif /* USE_WIN32 */
-    free(buff);
+    str_free(buff);
     return num;
 }
 
@@ -199,17 +187,13 @@ LPTSTR str2tstr(const LPSTR in) {
     len=MultiByteToWideChar(CP_ACP, 0, in, -1, NULL, 0);
     if(!len)
         return NULL;
-    out=malloc((len+1)*sizeof(WCHAR));
-    if(!out)
-        return NULL;
+    out=str_alloc((len+1)*sizeof(WCHAR));
     len=MultiByteToWideChar(CP_ACP, 0, in, -1, out, len);
     if(!len)
         return NULL;
 #else
     len=strlen(in);
-    out=malloc(len+1);
-    if(!out)
-        return NULL;
+    out=str_alloc(len+1);
     strcpy(out, in);
 #endif
     return out;
@@ -223,17 +207,13 @@ LPSTR tstr2str(const LPTSTR in) {
     len=WideCharToMultiByte(CP_ACP, 0, in, -1, NULL, 0, NULL, NULL);
     if(!len)
         return NULL;
-    out=malloc(len+1);
-    if(!out)
-        return NULL;
+    out=str_alloc(len+1);
     len=WideCharToMultiByte(CP_ACP, 0, in, -1, out, len, NULL, NULL);
     if(!len)
         return NULL;
 #else
     len=strlen(in);
-    out=malloc(len+1);
-    if(!out)
-        return NULL;
+    out=str_alloc(len+1);
     strcpy(out, in);
 #endif
     return out;
