@@ -1,6 +1,6 @@
 /*
  *   stunnel       Universal SSL tunnel
- *   Copyright (C) 1998-2011 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *   Copyright (C) 1998-2012 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -50,18 +50,25 @@ static LOG_MODE mode=LOG_MODE_NONE;
 
 #if !defined(USE_WIN32) && !defined(__vms)
 
+static int syslog_opened=0;
+
 void syslog_open(void) {
+    syslog_close();
     if(global_options.option.syslog)
 #ifdef __ultrix__
         openlog("stunnel", 0);
 #else
         openlog("stunnel", LOG_CONS|LOG_NDELAY, global_options.facility);
 #endif /* __ultrix__ */
+    syslog_opened=1;
 }
 
-void syslog_close(void) { /* this function is not currently used */
-    if(global_options.option.syslog)
-        closelog();
+void syslog_close(void) {
+    if(syslog_opened) {
+        if(global_options.option.syslog)
+            closelog();
+        syslog_opened=0;
+    }
 }
 
 #endif /* !defined(USE_WIN32) && !defined(__vms) */
@@ -92,6 +99,7 @@ void log_flush(LOG_MODE new_mode) {
     if(mode!=LOG_MODE_CONFIGURED)
         mode=new_mode;
 
+    enter_critical_section(CRIT_LOG);
     while(head) {
         log_raw(head->level, head->stamp, head->id, head->text);
         str_free(head->stamp);
@@ -100,6 +108,7 @@ void log_flush(LOG_MODE new_mode) {
         head=head->next;
         str_free(tmp);
     }
+    leave_critical_section(CRIT_LOG);
     head=tail=NULL;
 }
 
@@ -107,6 +116,7 @@ void s_log(int level, const char *format, ...) {
     va_list ap;
     char *text, *stamp, *id;
     struct LIST *tmp;
+    int libc_error, socket_error;
     time_t gmt;
     struct tm *timeptr;
 #if defined(HAVE_LOCALTIME_R) && defined(_REENTRANT)
@@ -117,6 +127,9 @@ void s_log(int level, const char *format, ...) {
     if(mode==LOG_MODE_CONFIGURED && level>global_options.debug_level)
         return;
 
+    libc_error=get_last_error();
+    socket_error=get_last_socket_error();
+
     time(&gmt);
 #if defined(HAVE_LOCALTIME_R) && defined(_REENTRANT)
     timeptr=localtime_r(&gmt, &timestruct);
@@ -133,6 +146,7 @@ void s_log(int level, const char *format, ...) {
     va_end(ap);
 
     if(mode==LOG_MODE_NONE) { /* save the text to log it later */
+        enter_critical_section(CRIT_LOG);
         tmp=str_alloc(sizeof(struct LIST));
         tmp->next=NULL;
         tmp->level=level;
@@ -144,12 +158,16 @@ void s_log(int level, const char *format, ...) {
         else
             head=tmp;
         tail=tmp;
+        leave_critical_section(CRIT_LOG);
     } else { /* ready log the text directly */
         log_raw(level, stamp, id, text);
         str_free(stamp);
         str_free(id);
         str_free(text);
     }
+
+    set_last_error(libc_error);
+    set_last_socket_error(socket_error);
 }
 
 static void log_raw(const int level, const char *stamp,
@@ -172,10 +190,18 @@ static void log_raw(const int level, const char *stamp,
 
     /* log the line to GUI/stderr */
 #ifdef USE_WIN32
-    if(mode==LOG_MODE_ERROR ||
+    if(mode==LOG_MODE_ERROR || /* always log to the GUI window */
             (mode==LOG_MODE_INFO && level<LOG_DEBUG) ||
             level<=global_options.debug_level)
-        win_log(line); /* always log to the GUI window */
+        SendMessage(hwnd, WM_LOG, (WPARAM)line, 0);
+#if 0
+    /* logging to Windows console for nogui.c */
+    LPTSTR tstr;
+
+    tstr=str2tstr(line);
+    RETAILMSG(TRUE, (TEXT("%s\r\n"), tstr));
+    str_free(tstr);
+#endif
 #else /* Unix */
     if(mode==LOG_MODE_ERROR || /* always log LOG_MODE_ERROR to stderr */
             (mode==LOG_MODE_INFO && level<LOG_DEBUG) ||
@@ -187,7 +213,7 @@ static void log_raw(const int level, const char *stamp,
     str_free(line);
 }
 
-/* memory allocation failed - it is not allowed to use any str.c functions */
+/* critical problem - str.c functions are not safe to use */
 void fatal(char *error, char *file, int line) {
     char text[80];
 #ifdef USE_WIN32
@@ -212,7 +238,7 @@ void fatal(char *error, char *file, int line) {
 #endif /* !USE_WIN32 */
 
     snprintf(text, sizeof text, /* without newline */
-        "INTERNAL ERROR: Out of memory at %s, line %d", file, line);
+        "INTERNAL ERROR: %s at %s, line %d", error, file, line);
 
 #if !defined(USE_WIN32) && !defined(__vms)
     if(global_options.option.syslog)
@@ -221,10 +247,10 @@ void fatal(char *error, char *file, int line) {
 
 #ifdef USE_WIN32
 #ifdef _WIN32_WCE
-    MessageBox(NULL, TEXT("INTERNAL ERROR: Out of memory"),
+    MessageBox(hwnd, TEXT("INTERNAL ERROR"),
         TEXT("stunnel"), MB_ICONERROR);
 #else /* _WIN32_WCE */
-    MessageBox(NULL, text, "stunnel", MB_ICONERROR);
+    MessageBox(hwnd, text, "stunnel", MB_ICONERROR);
 #endif /* _WIN32_WCE */
 #endif /* USE_WIN32 */
 
