@@ -1,6 +1,6 @@
 /*
  *   stunnel       Universal SSL tunnel
- *   Copyright (C) 1998-2011 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *   Copyright (C) 1998-2012 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -101,13 +101,15 @@ int s_poll_canwrite(s_poll_set *fds, int fd) {
     return 0;
 }
 
-int s_poll_error(s_poll_set *fds, int fd) {
+int s_poll_error(s_poll_set *fds, FD *s) {
     unsigned int i;
 
+    if(!s->is_socket)
+        return 0;
     for(i=0; i<fds->nfds; i++)
-        if(fds->ufds[i].fd==fd)
+        if(fds->ufds[i].fd==s->fd)
             return fds->ufds[i].revents&(POLLERR|POLLNVAL) ?
-                get_socket_error(fd) : 0;
+                get_socket_error(s->fd) : 0;
     return 0;
 }
 
@@ -217,7 +219,9 @@ int s_poll_wait(s_poll_set *fds, int sec, int msec) {
 
     /* it's illegal to deallocate the stack of the current context */
     if(to_free) { /* a delayed deallocation is scheduled */
+#ifdef DEBUG_UCONTEXT
         s_log(LOG_DEBUG, "Releasing context %ld", to_free->id);
+#endif
         str_free(to_free->stack);
         str_free(to_free);
         to_free=NULL;
@@ -245,15 +249,21 @@ int s_poll_wait(s_poll_set *fds, int sec, int msec) {
     /* switch threads */
     if(fds) { /* swap the current context */
         if(context->id!=ready_head->id) {
+#ifdef DEBUG_UCONTEXT
             s_log(LOG_DEBUG, "Context swap: %ld -> %ld",
                 context->id, ready_head->id);
+#endif
             swapcontext(&context->context, &ready_head->context);
+#ifdef DEBUG_UCONTEXT
             s_log(LOG_DEBUG, "Current context: %ld", ready_head->id);
+#endif
         }
         return ready_head->ready;
     } else { /* drop the current context */
+#ifdef DEBUG_UCONTEXT
         s_log(LOG_DEBUG, "Context set: %ld (dropped) -> %ld",
             context->id, ready_head->id);
+#endif
         setcontext(&ready_head->context);
         ioerror("setcontext"); /* should not ever happen */
         return 0;
@@ -308,10 +318,12 @@ int s_poll_canwrite(s_poll_set *fds, int fd) {
     return FD_ISSET(fd, &fds->owfds);
 }
 
-int s_poll_error(s_poll_set *fds, int fd) {
-    if(!FD_ISSET(fd, &fds->orfds)) /* error conditions are signaled as read */
+int s_poll_error(s_poll_set *fds, FD *s) {
+    if(!s->is_socket)
         return 0;
-    return get_socket_error(fd); /* check if it's really an error */
+    if(!FD_ISSET(s->fd, &fds->orfds)) /* error conditions are signaled as read */
+        return 0;
+    return get_socket_error(s->fd); /* check if it's really an error */
 }
 
 int s_poll_wait(s_poll_set *fds, int sec, int msec) {
@@ -342,8 +354,9 @@ int set_socket_options(int s, int type) {
     extern SOCK_OPT sock_opts[];
     static char *type_str[3]={"accept", "local", "remote"};
     int opt_size;
+    int retval=0; /* no error found */
 
-    for(ptr=sock_opts;ptr->opt_str;ptr++) {
+    for(ptr=sock_opts; ptr->opt_str; ptr++) {
         if(!ptr->opt_val[type])
             continue; /* default */
         switch(ptr->opt_type) {
@@ -368,14 +381,17 @@ int set_socket_options(int s, int type) {
                     ptr->opt_str, type_str[type]);
             } else {
                 sockerror(ptr->opt_str);
-                return -1; /* FAILED */
+                retval=-1; /* failed to set this option */
             }
-        } else {
+        }
+#ifdef DEBUG_FD_ALLOC
+        else {
             s_log(LOG_DEBUG, "Option %s set on %s socket",
                 ptr->opt_str, type_str[type]);
         }
+#endif /* DEBUG_FD_ALLOC */
     }
-    return 0; /* OK */
+    return retval; /* returns 0 when all options succeeded */
 }
 
 int get_socket_error(const int fd) {
@@ -383,7 +399,7 @@ int get_socket_error(const int fd) {
     socklen_t optlen=sizeof err;
 
     if(getsockopt(fd, SOL_SOCKET, SO_ERROR, (void *)&err, &optlen))
-        return get_last_socket_error(); /* failed -> ask why */
+        err=get_last_socket_error(); /* failed -> ask why */
     return err;
 }
 
@@ -426,16 +442,12 @@ int connect_blocking(CLI *c, SOCKADDR_UNION *addr, socklen_t addrlen) {
         str_free(dst);
         return -1;
     default:
-        if(s_poll_canread(c->fds, c->fd) || s_poll_error(c->fds, c->fd)) {
-            /* newly connected socket should not be ready for read */
-            /* get the resulting error code, now */
-            error=get_socket_error(c->fd);
-            if(error) { /* really an error? */
-                s_log(LOG_ERR, "connect_blocking: getsockopt %s: %s (%d)",
-                    dst, s_strerror(error), error);
-                str_free(dst);
-                return -1;
-            }
+        error=get_socket_error(c->fd);
+        if(error) {
+            s_log(LOG_ERR, "connect_blocking: connect %s: %s (%d)",
+               dst, s_strerror(error), error);
+            str_free(dst);
+            return -1;
         }
         if(s_poll_canwrite(c->fds, c->fd)) {
             s_log(LOG_NOTICE, "connect_blocking: connected %s", dst);
@@ -517,7 +529,7 @@ void read_blocking(CLI *c, int fd, void *ptr, int len) {
     }
 }
 
-void fdputline(CLI *c, int fd, const char *line) {
+void fd_putline(CLI *c, int fd, const char *line) {
     char *tmpline;
     const char crlf[]="\r\n";
     int len;
@@ -531,7 +543,7 @@ void fdputline(CLI *c, int fd, const char *line) {
     str_free(tmpline);
 }
 
-char *fdgetline(CLI *c, int fd) {
+char *fd_getline(CLI *c, int fd) {
     char *line=NULL, *tmpline;
     int ptr=0;
 
@@ -540,29 +552,29 @@ char *fdgetline(CLI *c, int fd) {
         s_poll_add(c->fds, fd, 1, 0); /* read */
         switch(s_poll_wait(c->fds, c->opt->timeout_busy, 0)) {
         case -1:
-            sockerror("fdgetline: s_poll_wait");
+            sockerror("fd_getline: s_poll_wait");
             str_free(line);
             longjmp(c->err, 1); /* error */
         case 0:
-            s_log(LOG_INFO, "fdgetline: s_poll_wait:"
+            s_log(LOG_INFO, "fd_getline: s_poll_wait:"
                 " TIMEOUTbusy exceeded: sending reset");
             str_free(line);
             longjmp(c->err, 1); /* timeout */
         case 1:
             break; /* OK */
         default:
-            s_log(LOG_ERR, "fdgetline: s_poll_wait: Unknown result");
+            s_log(LOG_ERR, "fd_getline: s_poll_wait: Unknown result");
             str_free(line);
             longjmp(c->err, 1); /* error */
         }
         line=str_realloc(line, ptr+1);
         switch(readsocket(fd, line+ptr, 1)) {
         case -1: /* error */
-            sockerror("fdgetline: readsocket");
+            sockerror("fd_getline: readsocket");
             str_free(line);
             longjmp(c->err, 1);
         case 0: /* EOF */
-            s_log(LOG_ERR, "fdgetline: Unexpected socket close");
+            s_log(LOG_ERR, "fd_getline: Unexpected socket close");
             str_free(line);
             longjmp(c->err, 1);
         }
@@ -573,7 +585,7 @@ char *fdgetline(CLI *c, int fd) {
         if(line[ptr]=='\0')
             break;
         if(++ptr>65536) { /* >64KB --> DoS protection */
-            s_log(LOG_ERR, "fdgetline: Line too long");
+            s_log(LOG_ERR, "fd_getline: Line too long");
             str_free(line);
             longjmp(c->err, 1);
         }
@@ -586,7 +598,7 @@ char *fdgetline(CLI *c, int fd) {
     return line;
 }
 
-void fdprintf(CLI *c, int fd, const char *format, ...) {
+void fd_printf(CLI *c, int fd, const char *format, ...) {
     va_list ap;
     char *line;
 
@@ -594,10 +606,10 @@ void fdprintf(CLI *c, int fd, const char *format, ...) {
     line=str_vprintf(format, ap);
     va_end(ap);
     if(!line) {
-        s_log(LOG_ERR, "fdprintf: str_vprintf failed");
+        s_log(LOG_ERR, "fd_printf: str_vprintf failed");
         longjmp(c->err, 1);
     }
-    fdputline(c, fd, line);
+    fd_putline(c, fd, line);
     str_free(line);
 }
 
