@@ -1,6 +1,6 @@
 /*
  *   stunnel       Universal SSL tunnel
- *   Copyright (C) 1998-2011 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *   Copyright (C) 1998-2012 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -39,21 +39,15 @@
 #include "prototypes.h"
 
     /* global OpenSSL initalization: compression, engine, entropy */
-static int init_compression(void);
-static int init_prng(void);
-static int add_rand_file(char *);
-#ifdef HAVE_OSSL_ENGINE_H
-static char *init_engine(void);
-#endif
+static int init_compression(GLOBAL_OPTIONS *);
+static int init_prng(GLOBAL_OPTIONS *);
+static int add_rand_file(GLOBAL_OPTIONS *, const char *);
 
 int cli_index, opt_index; /* to keep structure for callbacks */
 
 void ssl_init(void) { /* init SSL before parsing configuration file */
-#ifdef HAVE_OPENSSL
-    OpenSSL_add_all_algorithms();
-#endif
-    SSLeay_add_ssl_algorithms();
     SSL_load_error_strings();
+    SSL_library_init();
     cli_index=SSL_get_ex_new_index(0, "cli index", NULL, NULL, NULL);
     opt_index=SSL_CTX_get_ex_new_index(0, "opt index", NULL, NULL, NULL);
 #ifdef HAVE_OSSL_ENGINE_H
@@ -61,60 +55,100 @@ void ssl_init(void) { /* init SSL before parsing configuration file */
 #endif
 }
 
-int ssl_configure(void) { /* configure global SSL settings */
+int ssl_configure(GLOBAL_OPTIONS *global) { /* configure global SSL settings */
 #ifdef USE_FIPS
-    if(FIPS_mode()!=global_options.option.fips) {
+    if(FIPS_mode()!=global->option.fips) {
         RAND_set_rand_method(NULL); /* reset RAND methods */
-        if(!FIPS_mode_set(global_options.option.fips)) {
+        if(!FIPS_mode_set(global->option.fips)) {
             ERR_load_crypto_strings();
             sslerror("FIPS_mode_set");
-            return 0;
+            return 1;
         }
-        s_log(LOG_NOTICE, "FIPS mode %s",
-            global_options.option.fips ? "enabled" : "disabled");
     }
+    s_log(LOG_NOTICE, "FIPS mode is %s",
+        global->option.fips ? "enabled" : "disabled");
 #endif /* USE_FIPS */
-    if(global_options.compression!=COMP_NONE && !init_compression())
-        return 0;
-    if(!init_prng())
-        return 0;
+    if(init_compression(global))
+        return 1;
+    if(init_prng(global))
+        return 1;
     s_log(LOG_DEBUG, "PRNG seeded successfully");
-    return 1; /* SUCCESS */
+    return 0; /* SUCCESS */
 }
 
-static int init_compression(void) {
-    int id=0;
-    COMP_METHOD *cm=NULL;
-    char *name="unknown";
+static int init_compression(GLOBAL_OPTIONS *global) {
+#ifndef OPENSSL_NO_COMP
+    SSL_COMP *comp;
+    STACK_OF(SSL_COMP) *ssl_comp_methods;
 
-    switch(global_options.compression) {
-    case COMP_ZLIB:
-        id=0xe0;
-        cm=COMP_zlib();
-        name="zlib";
-        break;
-    case COMP_RLE:
-        id=0xe1;
-        cm=COMP_rle();
-        name="rle";
-        break;
-    default:
-        s_log(LOG_ERR, "INTERNAL ERROR: Bad compression method");
+    ssl_comp_methods=SSL_COMP_get_compression_methods();
+    if(!ssl_comp_methods) {
+        s_log(LOG_ERR, "Failed to get compression methods");
+        return 1;
+    }
+
+    /* delete OpenSSL defaults (empty the SSL_COMP stack) */
+    /* cannot use sk_SSL_COMP_pop_free, as it also destroys the stack itself */
+    while(sk_SSL_COMP_num(ssl_comp_methods))
+        OPENSSL_free(sk_SSL_COMP_pop(ssl_comp_methods));
+
+    if(global->compression==COMP_NONE) {
+        s_log(LOG_DEBUG, "Compression not enabled");
+        return 0; /* success */
+    }
+
+    /* insert RFC 1951 (DEFLATE) algoritm */
+    if(SSLeay()>=0x00908051L) { /* 0.9.8e-beta1 */
+        /* only allow DEFLATE with OpenSSL 0.9.8 or later
+           with openssl #1468 zlib memory leak fixed */
+        comp=(SSL_COMP *)OPENSSL_malloc(sizeof(SSL_COMP));
+        if(!comp) {
+            s_log(LOG_ERR, "OPENSSL_malloc filed");
+            return 1;
+        }
+        comp->id=1; /* RFC 1951 */
+        comp->method=COMP_zlib();
+        if(!comp->method || comp->method->type==NID_undef) {
+            OPENSSL_free(comp);
+            s_log(LOG_ERR, "Failed to initialize compression method");
+            return 1;
+        }
+        comp->name=comp->method->name;
+        sk_SSL_COMP_push(ssl_comp_methods, comp);
+    }
+
+    /* also insert one of obsolete (ZLIB/RLE) algoritms */
+    comp=(SSL_COMP *)OPENSSL_malloc(sizeof(SSL_COMP));
+    if(!comp) {
+        s_log(LOG_ERR, "OPENSSL_malloc filed");
+        return 1;
+    }
+    if(global->compression==COMP_ZLIB) {
+        comp->id=0xe0; /* 224 - within private range (193 to 255) */
+        comp->method=COMP_zlib();
+    } else if(global->compression==COMP_RLE) {
+        comp->id=0xe1; /* 225 - within private range (193 to 255) */
+        comp->method=COMP_rle();
+    } else {
+        s_log(LOG_INFO, "Compression enabled: %d algorithm(s)",
+            sk_SSL_COMP_num(ssl_comp_methods));
+        OPENSSL_free(comp);
         return 0;
     }
-    if(!cm || cm->type==NID_undef) {
-        s_log(LOG_ERR, "Failed to initialize %s compression method", name);
-        return 0;
+    if(!comp->method || comp->method->type==NID_undef) {
+        OPENSSL_free(comp);
+        s_log(LOG_ERR, "Failed to initialize compression method");
+        return 1;
     }
-    if(SSL_COMP_add_compression_method(id, cm)) {
-        s_log(LOG_ERR, "Failed to add %s compression method", name);
-        return 0;
-    }
-    s_log(LOG_INFO, "Compression enabled using %s method", name);
-    return 1;
+    comp->name=comp->method->name;
+    sk_SSL_COMP_push(ssl_comp_methods, comp);
+    s_log(LOG_INFO, "Compression enabled: %d algorithm(s)",
+        sk_SSL_COMP_num(ssl_comp_methods));
+#endif /* OPENSSL_NO_COMP */
+    return 0; /* success */
 }
 
-static int init_prng(void) {
+static int init_prng(GLOBAL_OPTIONS *global) {
     int totbytes=0;
     char filename[256];
     int bytes;
@@ -125,74 +159,74 @@ static int init_prng(void) {
 
     /* if they specify a rand file on the command line we
        assume that they really do want it, so try it first */
-    if(global_options.rand_file) {
-        totbytes+=add_rand_file(global_options.rand_file);
+    if(global->rand_file) {
+        totbytes+=add_rand_file(global, global->rand_file);
         if(RAND_status())
-            return 1;
+            return 0; /* success */
     }
 
     /* try the $RANDFILE or $HOME/.rnd files */
     RAND_file_name(filename, 256);
     if(filename[0]) {
-        totbytes+=add_rand_file(filename);
+        totbytes+=add_rand_file(global, filename);
         if(RAND_status())
-            return 1;
+            return 0; /* success */
     }
 
 #ifdef RANDOM_FILE
-    totbytes+=add_rand_file(RANDOM_FILE);
+    totbytes+=add_rand_file(global, RANDOM_FILE);
     if(RAND_status())
-        return 1;
+        return 0; /* success */
 #endif
 
 #ifdef USE_WIN32
     RAND_screen();
     if(RAND_status()) {
         s_log(LOG_DEBUG, "Seeded PRNG with RAND_screen");
-        return 1;
+        return 0; /* success */
     }
     s_log(LOG_DEBUG, "RAND_screen failed to sufficiently seed PRNG");
 #else
-    if(global_options.egd_sock) {
-        if((bytes=RAND_egd(global_options.egd_sock))==-1) {
-            s_log(LOG_WARNING, "EGD Socket %s failed", global_options.egd_sock);
+    if(global->egd_sock) {
+        if((bytes=RAND_egd(global->egd_sock))==-1) {
+            s_log(LOG_WARNING, "EGD Socket %s failed", global->egd_sock);
             bytes=0;
         } else {
             totbytes+=bytes;
             s_log(LOG_DEBUG, "Snagged %d random bytes from EGD Socket %s",
-                bytes, global_options.egd_sock);
-            return 1; /* OpenSSL always gets what it needs or fails,
+                bytes, global->egd_sock);
+            return 0; /* OpenSSL always gets what it needs or fails,
                          so no need to check if seeded sufficiently */
         }
     }
 #endif /* USE_WIN32 */
 
     /* try the good-old default /dev/urandom, if available  */
-    totbytes+=add_rand_file("/dev/urandom");
+    totbytes+=add_rand_file(global, "/dev/urandom");
     if(RAND_status())
-        return 1;
+        return 0; /* success */
 
     /* random file specified during configure */
     s_log(LOG_ERR, "PRNG seeded with %d bytes total", totbytes);
     s_log(LOG_ERR, "PRNG was not seeded with enough random bytes");
-    return 0; /* FAILED */
+    return 1; /* FAILED */
 }
 
-static int add_rand_file(char *filename) {
+static int add_rand_file(GLOBAL_OPTIONS *global, const char *filename) {
     int readbytes;
     int writebytes;
     struct stat sb;
 
     if(stat(filename, &sb))
-        return 0;
-    if((readbytes=RAND_load_file(filename, global_options.random_bytes)))
+        return 0; /* could not stat() file  -> return 0 bytes */
+    if((readbytes=RAND_load_file(filename, global->random_bytes)))
         s_log(LOG_DEBUG, "Snagged %d random bytes from %s",
             readbytes, filename);
     else
         s_log(LOG_INFO, "Unable to retrieve any random data from %s",
             filename);
     /* write new random data for future seeding if it's a regular file */
-    if(global_options.option.rand_write && (sb.st_mode & S_IFREG)){
+    if(global->option.rand_write && (sb.st_mode & S_IFREG)){
         writebytes=RAND_write_file(filename);
         if(writebytes==-1)
             s_log(LOG_WARNING, "Failed to write strong random data to %s - "
@@ -204,85 +238,4 @@ static int add_rand_file(char *filename) {
     return readbytes;
 }
 
-#ifdef HAVE_OSSL_ENGINE_H
-
-#define MAX_ENGINES 256
-static ENGINE *engines[MAX_ENGINES]; /* table of engines */
-static int current_engine=0;
-static int engine_initialized;
-
-char *open_engine(const char *name) {
-    s_log(LOG_DEBUG, "Enabling support for engine '%s'", name);
-    if(!strcasecmp(name, "auto")) {
-        ENGINE_register_all_complete();
-        s_log(LOG_DEBUG, "Auto engine support enabled");
-        return NULL; /* OK */
-    }
-
-    close_engine(); /* close the previous one (if specified) */
-    engines[current_engine]=ENGINE_by_id(name);
-    engine_initialized=0;
-    if(!engines[current_engine]) {
-        sslerror("ENGINE_by_id");
-        return "Failed to open the engine";
-    }
-    return NULL; /* OK */
-}
-
-char *ctrl_engine(const char *cmd, const char *arg) {
-    if(!strcasecmp(cmd, "INIT")) { /* special control command */
-        return init_engine();
-    }
-    if(arg)
-        s_log(LOG_DEBUG, "Executing engine control command %s:%s", cmd, arg);
-    else
-        s_log(LOG_DEBUG, "Executing engine control command %s", cmd);
-    if(!ENGINE_ctrl_cmd_string(engines[current_engine], cmd, arg, 0)) {
-        sslerror("ENGINE_ctrl_cmd_string");
-        return "Failed to execute the engine control command";
-    }
-    return NULL; /* OK */
-}
-
-void close_engine(void) {
-    if(!engines[current_engine])
-        return; /* no engine was opened -> nothing to do */
-    init_engine();
-    ++current_engine;
-#if 0
-    ENGINE_finish(e);
-    ENGINE_free(e);
-    e=NULL;
-    s_log(LOG_DEBUG, "Engine closed");
-#endif
-}
-
-static char *init_engine(void) {
-    if(engine_initialized)
-        return NULL; /* OK */
-    engine_initialized=1;
-    s_log(LOG_DEBUG, "Initializing engine %d", current_engine+1);
-    if(!ENGINE_init(engines[current_engine])) {
-        if(ERR_peek_last_error()) /* really an error */
-            sslerror("ENGINE_init");
-        else
-            s_log(LOG_ERR, "Engine %d not initialized", current_engine+1);
-        return "Engine initialization failed";
-    }
-    if(!ENGINE_set_default(engines[current_engine], ENGINE_METHOD_ALL)) {
-        sslerror("ENGINE_set_default");
-        return "Selecting default engine failed";
-    }
-    s_log(LOG_DEBUG, "Engine %d initialized", current_engine+1);
-    return NULL; /* OK */
-}
-
-ENGINE *get_engine(int i) {
-    if(i<1 || i>current_engine)
-        return NULL;
-    return engines[i-1];
-}
-
-#endif /* HAVE_OSSL_ENGINE_H */
-
 /* end of ssl.c */
